(***********************************************************************)
(*                                                                     *)
(*          Run-time Code Generation Library for Objective Caml        *)
(*                                                                     *)
(*                   Dmitri Lomov (dsl@tepkom.ru)                      *)
(*              St-Petersburg State University, Russia                 *)
(*                                                                     *)
(***********************************************************************)    
(* $Id: bitSet.ml,v 1.1.1.1 2002/02/28 11:11:01 mitya Exp $ *)
type t = int array

(* 16 bits in each element *)

let make n =
  let len = 
    if (n land 0xF) == 0
    then (n lsr 4)
    else (n lsr 4) + 1 in
  Array.make len 0

let is s n =
  ((Array.unsafe_get s (n lsr 4)) land (1 lsl (n land 0xF))) != 0

let on s n =
  let elem = Array.unsafe_get s (n lsr 4) in
  Array.unsafe_set s (n lsr 4) (elem lor (1 lsl (n land 0xF)))

let off s n =
  let elem = Array.unsafe_get s (n lsr 4) in
  Array.unsafe_set s (n lsr 4) (elem land (lnot (1 lsl (n land 0xF))))

let dup s = (Obj.magic (Obj.dup (Obj.repr s)))

let union s1 s2 =
  let dlen = Array.length s1 - 1 in
  for i = 0 to dlen do
    s1.(i) <- s1.(i) lor s2.(i)
  done

let sub s1 s2 =
  let dlen = Array.length s1 - 1 in
  for i = 0 to dlen do
    s1.(i) <- s1.(i) land  (lnot s2.(i))
  done

