(* bytetypes.ml *)
(* $Header: /home/mitya/cvsroot/dml/bytecode/bytecode.ml,v 1.2 2002/04/03 21:58:47 mitya Exp $ *)
open Opcodes

type comparison = LE | GE | LT | GT | EQ | NEQ

type label = int

type instruction = 
    Ilabel of label
  | IVequal 
  | Iatom of int
  | Iacc of int
  | Ienvacc of int
  | Ipush
  | Ipop of int
  | Iassign of int
  | Ipush_retaddr of label
  | Iapply of int                       
  | Iappterm of int * int               
  | Ireturn of int                      
  | Irestart
  | Igrab of int                        
  | Iclosure of int
  | Iclosurerec of label list * int
  | Ioffsetclosure of int
  | Iconst of int
  | Imakeblock of int * int  (* size * tag *)           
  | Imakefloatblock of int
  | Igetfield of int
  | Isetfield of int
  | Igetfloatfield of int
  | Isetfloatfield of int
  | Ivectlength
  | Igetvectitem
  | Isetvectitem
  | IVgetvectitem
  | IVsetvectitem
  | IVgetfloatitem
  | IVsetfloatitem
  | Igetstringchar
  | Isetstringchar
  | Ibranch of label
  | Ibranchif of label
  | Ibranchifnot of label
  | Ibranchintcmp of comparison * int * label
(*  | Iswitch of label array * label array*)
  | Iboolnot
  | Ipushtrap of label
  | Ipoptrap
  | Iraise
  | Icheck_signals
  | IVreg_closure
  | IVrec_reg_closure
  | Inegint 
  | Iaddint 
  | Isubint 
  | Imulint 
  | Idivint 
  | Imodint
  | Iandint 
  | Iorint 
  | Ixorint 
  | Ilslint 
  | Ilsrint 
  | Iasrint
  | Iintcomp of comparison
  | IVcompare of comparison
  | Ioffsetint of int
  | Ioffsetref of int
  | Igetmethod
  | Istop
  | IVflop of Code.flop
  | IVfloat_neg

let get_0ary_opcode  = function 
| Ipush -> Opcodes.opPUSH
| Irestart -> Opcodes.opRESTART
| Ivectlength -> Opcodes.opVECTLENGTH
| Igetvectitem -> Opcodes.opGETVECTITEM
| Isetvectitem -> Opcodes.opSETVECTITEM
| Igetstringchar -> Opcodes.opGETSTRINGCHAR
| Isetstringchar -> Opcodes.opSETSTRINGCHAR
| Iboolnot -> Opcodes.opBOOLNOT
| Ipoptrap -> Opcodes.opPOPTRAP
| Iraise -> Opcodes.opRAISE
| Icheck_signals -> Opcodes.opCHECK_SIGNALS
| Inegint -> Opcodes.opNEGINT
| Iaddint -> Opcodes.opADDINT
| Isubint -> Opcodes.opSUBINT
| Imulint -> Opcodes.opMULINT
| Idivint -> Opcodes.opDIVINT
| Imodint -> Opcodes.opMODINT
| Iandint -> Opcodes.opANDINT
| Iorint -> Opcodes.opORINT
| Ixorint -> Opcodes.opXORINT
| Ilslint -> Opcodes.opLSLINT
| Ilsrint -> Opcodes.opLSRINT
| Iasrint -> Opcodes.opASRINT
| (Iintcomp LE) -> Opcodes.opLEINT
| (Iintcomp LT) -> Opcodes.opLTINT
| (Iintcomp NEQ) -> Opcodes.opNEQ
| (Iintcomp EQ) -> Opcodes.opEQ
| (Iintcomp GE) -> Opcodes.opGEINT
| (Iintcomp GT) -> Opcodes.opGTINT
| Igetmethod -> Opcodes.opGETMETHOD
| Istop -> Opcodes.opSTOP
| _ -> 0

let not_comparison = function
    EQ -> NEQ
  | NEQ -> EQ
  | LT -> GE
  | LE -> GT
  | GT -> LE
  | GE -> LT

let commute_comparison = function
    EQ -> EQ
  | NEQ -> NEQ
  | LT -> GT
  | LE -> GE
  | GT -> LT
  | GE -> LE

let comparison_from_code = function
    Code.EQ -> EQ
  | Code.NEQ -> NEQ
  | Code.LE -> LE
  | Code.GE -> GE
  | Code.LT -> LT
  | Code.GT -> GT







