(***********************************************************************)
(*                                                                     *)
(*          Run-time Code Generation Library for Objective Caml        *)
(*                                                                     *)
(*                   Dmitri Lomov (dsl@tepkom.ru)                      *)
(*              St-Petersburg State University, Russia                 *)
(*                                                                     *)
(***********************************************************************)    

(* $Header: /home/mitya/cvsroot/dml/bytecode/env.ml,v 1.1.1.1 2002/02/28 11:11:01 mitya Exp $ *)

open Array

type 'a node = 
  Empty
| Node of 'a*('a node)

let hash_size = 32

type ('a,'k) t = 
{
 elems : ('a node) array;
 hash : ('k -> int);
 key : ('a -> 'k);
}

type 'a transformator = 'a -> 'a


let init hash' key' hash_size =
{
  elems = init hash_size (fun _ -> Empty);
  hash = hash';
  key = key';
}
    

let get e i =
  let rec search n =
    match n with
      Empty -> None
    | Node(a, next) ->
        if (e.key a) = i
        then Some a
        else search next in
  search (e.elems.(e.hash i))  

let add x e =
  let k = e.key x in
  let rec add_to_list = function
    Empty -> Node(x, Empty)
  | Node(h, t) ->
      if e.key h = k
      then Node(x,t)
      else Node(h, add_to_list t) in
  let h = e.hash k in 
  e.elems.(h) <- (add_to_list e.elems.(h)); e

let fold e b f =
  let rec fold_elem b n =
    match n with 
      Empty -> b
    | Node(x,next) ->
        fold_elem (f b x) next in
  fold_left fold_elem b e.elems
  
let transform k f e =
  let rec aux_t = function 
    Empty -> Empty
  | Node(h,t) ->
      if e.key h = k
      then Node((f h), t)
      else Node(h, (aux_t t)) in
  let h = e.hash k in
  e.elems.(h) <- (aux_t e.elems.(h)); e
