
(***********************************************************************)
(*                                                                     *)
(*          Run-time Code Generation Library for Objective Caml        *)
(*                                                                     *)
(*                   Dmitri Lomov (dsl@tepkom.ru)                      *)
(*              St-Petersburg State University, Russia                 *)
(*                                                                     *)
(***********************************************************************)    
(* $Id: rawBytecode.ml,v 1.2 2002/04/03 22:01:43 mitya Exp $ *)
type t
type code_pointer

external raw_register_closure : int -> Obj.t -> Obj.t =
  "raw_register_closure"
external raw_rec_register_closure : int -> Obj.t -> Obj.t =
  "raw_rec_register_closure"
external get_raw_register_closure_index : unit -> int = 
  "raw_get_raw_register_closure_index"
external get_raw_rec_register_closure_index : unit -> int = 
  "raw_get_raw_rec_register_closure_index"

external put_data : int -> t -> t = "raw_put_data"
external put_opcode : int -> t -> t = "raw_put_opcode"
external copy_code : code_pointer -> int -> t -> t = "raw_copy_code"
external put_data_at : int -> int -> t -> t = "raw_put_data_at"
external get_offset : t -> int = "raw_get_offset"
external raw_get_size : t -> int = "raw_get_size"

external raw_create : int -> t = "raw_create"
external raw_finish : t -> int -> int -> Obj.t list -> int = "raw_finish"

external raw_init : int -> int -> int -> unit =
  "raw_init"
external raw_uninit : unit -> unit =
  "raw_uninit"


let max_refs = 256
let max_code_blocks = 256
let code_area_size = 32767

let init () =
  raw_init max_refs max_code_blocks code_area_size

let uninit () = raw_uninit ()

let create len = raw_create len
let finish block offset n_ext ext_list = 
  raw_finish block offset n_ext ext_list

exception ObjIsNotClosure

let register_closure block closure =
  if (Obj.tag closure = Obj.closure_tag)
  then raw_register_closure block closure
  else raise ObjIsNotClosure

let rec_register_closure block closure =
  if (Obj.tag closure = Obj.closure_tag)
  then raw_rec_register_closure block closure
  else raise ObjIsNotClosure

exception WrongOffset

external raw_test : unit -> unit = "raw_test";;
external raw_gc : unit -> unit = "raw_gc";;

(*
    Printf.printf "Testing starts!\n"; flush stdout;
  init ();
  create 12 10;
  let b = create 256 10 in
  let z = create 12 10 in
  Gc.full_major ();
  raw_test ();
  let handle = finish b 0 0 [] in
  raw_test ();  
  raw_gc ();
  raw_test ();

  let d = create 12 10 in
  (let c = Obj.new_block Obj.closure_tag 3 in 
  register_closure handle c;);
  let d_h = finish d 0 1 [(Obj.repr 1)] in
  let c = Obj.new_block Obj.closure_tag 3 in
  register_closure d_h c;
  raw_test ();
  Gc.full_major ();
  raw_test ();
  raw_gc ();
  raw_test ();
  Printf.printf "raw_register_closure index is %d\n" 
    (get_raw_register_closure_index ());
  Printf.printf "----------------------------------------------\n";
  flush stdout;
  put_data Opcodes.opENVACC1 z;
  put_data Opcodes.opGETFIELD0 z;
  put_data Opcodes.opRETURN z;
  put_data 1 z;
  Printf.printf "Here we are!\n"; flush stdout;
  let z_h = finish z 0 1 [(Obj.repr 27)] in
  let f : int -> int =
    (let c = Obj.new_block Obj.closure_tag 3 in
    register_closure z_h c; (Obj.magic c)) in
  Printf.printf "Here we are!\n"; flush stdout;
  raw_test ();
  Printf.printf "%d %d %d\n" (f 1) (f 2) (f (f 3)); flush stdout;
  Gc.full_major ();
  raw_gc ();
  Printf.printf "%d %d %d\n" (f 1) (f 2) (f (f 3)); 
  uninit()
*)








