(***********************************************************************)
(*                                                                     *)
(*          Run-time Code Generation Library for Objective Caml        *)
(*                                                                     *)
(*                   Dmitri Lomov (dsl@tepkom.ru)                      *)
(*              St-Petersburg State University, Russia                 *)
(*                                                                     *)
(***********************************************************************)    

(* $Header: /home/mitya/cvsroot/dml/bytecode/rawBytecode.mli,v 1.1.1.1 2002/02/28 11:11:01 mitya Exp $ *)

type t
type code_pointer

(* Initialization and termination *)
val init : unit -> unit
val uninit : unit -> unit

(* Allocation *)
val create : int -> t
val finish : t -> int -> int -> Obj.t list -> int

(*
 * This function is O'Caml wrapper around 
 * raw_register_closure - C primitive that
 * is used every time closure with RTCG function is
 * created.
 *)
val register_closure : int -> Obj.t -> Obj.t
val rec_register_closure : int -> Obj.t -> Obj.t
exception ObjIsNotClosure
(*
 * returns index of C primitive raw_register_closure
 *)
val get_raw_register_closure_index : unit -> int
val get_raw_rec_register_closure_index : unit -> int


(* Data writers *)
(* data -> code -> code *)
val put_data : int -> t -> t
val put_opcode : int -> t -> t
val copy_code : code_pointer -> int -> t -> t

(* code -> offset *)
val get_offset : t -> int

(* data -> offset -> code -> code *)
val put_data_at : int -> int -> t -> t
exception WrongOffset
