(***********************************************************************)
(*                                                                     *)
(*          Run-time Code Generation Library for Objective Caml        *)
(*                                                                     *)
(*                   Dmitri Lomov (dsl@tepkom.ru)                      *)
(*              St-Petersburg State University, Russia                 *)
(*                                                                     *)
(***********************************************************************)    

(* $Id: code.ml,v 1.3 2002/04/03 22:06:16 mitya Exp $ *)

type any = int

type int_bin_op = 
    ISUB | IADD | IMUL | IDIV | IMOD |
    IAND | IOR  | IXOR |
    ILSL | ILSR | IASR 

type flop =
    FO_ADD | FO_SUB | FO_MUL | FO_DIV | FO_MOD | FO_POWER

type int_un_op = INEG | INOT

type comparison = LT | GT | LE | GE | EQ | NEQ

type id = int

type direction = DTo | DDownTo

type 'a code =
  Quote of 'a 
| Var of id
| Abstr of (any code -> 'a code)
| Fix of ('a code -> 'a code)
| Apply of (any -> 'a) code * any code
| Let of any code * (any code -> 'a code)

| If of bool code * 'a code * 'a code
| Bor of bool code * bool code
| Band of bool code * bool code
| Bnot of bool code

| Tuple of Obj.t 
| Slice of any code * int 

| ArraySet of any array code * int code * 'a code
| ArrayGet of 'a array code * int code
| ArrayLength of any array code

| StringSet of string code * int code * char code
| StringGet of string code * int code

| Ibin of int code * int_bin_op * int code 
| Iun of int_un_op * int code
| Icmp of int code * comparison * int code

| Flop of float code * flop * float code
| Float_neg of float code

| Aeq of any code * any code
| Aneq of any code * any code
| Equal of any code * any code
| Compare of any code * comparison * any code

| Ref of any code
| RefVal of 'a ref code
| RefSet of ('a ref code) * ('a code)
 
| For of int code * direction * int code * (int code -> 'a code)
| While of bool code * unit code
| Seq of unit code * 'a code




